From 95fa61c5928711bf9761f976cb3bbe68cd350234 Mon Sep 17 00:00:00 2001
From: =?utf-8?q?Rafa=C5=82=20Mi=C5=82ecki?= <zajec5@gmail.com>
Date: Sun, 5 Jul 2009 21:16:46 +0200
Subject: [PATCH] Add RHD_CARD_FLAG_FORCEBL to use on cards not reporting backlight control

Some manufactures use on chip backlight control, but enabling bit is not set by
default. This adds a way of flagging such devices.
---
 src/rhd_card.h  |    5 +++--
 src/rhd_dig.c   |    7 +++++--
 src/rhd_lvtma.c |    6 +++++-
 3 files changed, 13 insertions(+), 5 deletions(-)

diff --git a/src/rhd_card.h b/src/rhd_card.h
index d854931..f8aabab 100644
--- a/src/rhd_card.h
+++ b/src/rhd_card.h
@@ -37,10 +37,11 @@ struct rhdConnectorInfo {
 
 /* Some card specific flags, where and when needed */
 enum rhdCardFlag {
-    RHD_CARD_FLAG_NONE  = 0,
+    RHD_CARD_FLAG_NONE = 0,
     RHD_CARD_FLAG_DMS59 = 1, /* DMS59 connector is only reported as two DVI-I */
     RHD_CARD_FLAG_HPDSWAP = 2, /* some cards have broken connector tables */
-    RHD_CARD_FLAG_HPDOFF = 4 /* some have *very* broken connector tables */
+    RHD_CARD_FLAG_HPDOFF = 4, /* some have *very* broken connector tables */
+    RHD_CARD_FLAG_FORCEBL = 8 /* some cards don't have backlight bit enabled */
 };
 
 struct rhdCard {
diff --git a/src/rhd_dig.c b/src/rhd_dig.c
index 1c86ad0..5840507 100644
--- a/src/rhd_dig.c
+++ b/src/rhd_dig.c
@@ -46,6 +46,7 @@
 #include "rhd_atombios.h"
 #include "rhd_atomout.h"
 #endif
+#include "rhd_card.h"
 
 #define FMT2_OFFSET 0x800
 #define DIG1_OFFSET 0x000
@@ -1312,8 +1313,10 @@ GetLVDSInfo(RHDPtr rhdPtr, struct DIGPrivate *Private)
 			   & RV62_LVDS_24BIT_ENABLE) != 0);
 
     tmp = RHDRegRead(rhdPtr, RV620_LVTMA_BL_MOD_CNTL);
-    if (tmp & 0x1)
-	Private->BlLevel = ( tmp >> LVTMA_BL_MOD_LEVEL_SHIFT )  & 0xff;
+    if (tmp & LVTMA_BL_MOD_EN)
+	Private->BlLevel = (tmp >> LVTMA_BL_MOD_LEVEL_SHIFT) & 0xff;
+    else if (rhdPtr->Card && rhdPtr->Card->flags & RHD_CARD_FLAG_FORCEBL)
+	Private->BlLevel = 255;
     else
 	Private->BlLevel = -1;
 
diff --git a/src/rhd_lvtma.c b/src/rhd_lvtma.c
index b24bf5f..c79d1e9 100644
--- a/src/rhd_lvtma.c
+++ b/src/rhd_lvtma.c
@@ -52,6 +52,7 @@
 #include "rhd_atombios.h"
 #include "rhd_atomout.h"
 #endif
+#include "rhd_card.h"
 
 /*
  * First of all, make it more managable to code for both R500 and R600, as
@@ -615,9 +616,12 @@ LVDSInfoRetrieve(RHDPtr rhdPtr)
     tmp = RHDRegRead(rhdPtr, LVTMA_PWRSEQ_REF_DIV);
     Private->PowerRefDiv = tmp & 0x0FFF;
     Private->BlonRefDiv = (tmp >> 16) & 0x0FFF;
+
     tmp = RHDRegRead(rhdPtr, LVTMA_BL_MOD_CNTL);
-    if (tmp & 0x1)
+    if (tmp & LVTMA_BL_MOD_EN)
 	Private->BlLevel = (tmp >> 8) & 0xff;
+    else if (rhdPtr->Card && rhdPtr->Card->flags & RHD_CARD_FLAG_FORCEBL)
+	Private->BlLevel = 255;
     else
 	Private->BlLevel = -1; /* Backlight control seems to be done some other way */
 
-- 
1.6.0.2

